/**
  ******************************************************************************
  * @file    demo.h
  * @brief   This file contains functions prototypes for the demo.c file.
  * @author  STMicroelectronics - MCD Application Team
  * @version V1.2.0
  * @date    04-NOV-2010
  ******************************************************************************
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2010 STMicroelectronics</center></h2>
  * @image html logo.bmp
  ******************************************************************************
  */

// Define to prevent recursive inclusion -------------------------------------
#ifndef __DEMO_H
#define __DEMO_H

// Includes ------------------------------------------------------------------
#include "functions.h"
#include "mono_lcd.h"
#include "joystick_button.h"
#include "menu.h"
#include "STM8_TSL_API.h"
#include "STM8_TSL_Services.h"
#include "stm8l15x.h"

// Exported types ------------------------------------------------------------

// Exported constants --------------------------------------------------------

// Menus min/max values 
#define SLIDER_WHEEL_RESOLUTION_MIN  (0)
#define SLIDER_WHEEL_RESOLUTION_MAX  (7)
#define LP_DELAY_FACTOR_MIN          (2)
#define LP_DELAY_FACTOR_MAX         (12)
#define DXS_SETTING_MIN              (0)
#define DXS_SETTING_MAX              (3)
#define DTO_SETTING_MIN              (0)
#define DTO_SETTING_MAX             (20)
#define DEBOUNCE_SETTING_MIN         (1)
#define DEBOUNCE_SETTING_MAX         (9)
#define DETECT_TH_MIN                (2)
#define DETECT_TH_MAX              (127)
#define END_DETECT_TH_MIN            (1)
#define END_DETECT_TH_MAX          (126)
#define RECALIB_TH_MIN               (1) // absolute value
#define RECALIB_TH_MAX             (127) // absolute value

// Single-channel key structure to physical key mapping (Warning also defined in demo.c 
#if defined(BOARD_STMT_8L_EV1)
#define KEY0   sSCKeyInfo[5]
#define KEY1   sSCKeyInfo[0]
#define KEY2   sSCKeyInfo[9]
#define KEY3   sSCKeyInfo[4]
#define KEY4   sSCKeyInfo[8]
#define KEY5   sSCKeyInfo[3]
#define KEY6   sSCKeyInfo[7]
#define KEY7   sSCKeyInfo[2]
#define KEY8   sSCKeyInfo[6]
#define KEY9   sSCKeyInfo[1]
#define WHEEL  sMCKeyInfo[0]
#define SLIDER sMCKeyInfo[1]
#elif defined(BOARD_STM8L151CxT_TS1)
#define KEY0   sSCKeyInfo[5]
#define KEY1   sSCKeyInfo[7]
#define KEY2   sSCKeyInfo[1]
#define KEY3   sSCKeyInfo[0]
#define KEY4   sSCKeyInfo[6]
#define KEY5   sSCKeyInfo[9]
#define KEY6   sSCKeyInfo[2]
#define KEY7   sSCKeyInfo[4]
#define KEY8   sSCKeyInfo[8]
#define KEY9   sSCKeyInfo[3]
#define WHEEL  sMCKeyInfo[0]
#define SLIDER sMCKeyInfo[1]
#else
#error "Board not defined"
#endif

// Exported macro ------------------------------------------------------------
// Private functions ---------------------------------------------------------
// Exported variables --------------------------------------------------------

// Exported functions --------------------------------------------------------
void DETECTION_TIMEOUT_Exec(void);
void DEC_DETECTION_TIMEOUT_Exec(void);
void INC_DETECTION_TIMEOUT_Exec(void);

void DEBOUNCE_FILTER_Exec(void);
void DEC_DEBOUNCE_FILTER_Exec(void);
void INC_DEBOUNCE_FILTER_Exec(void);

void POWER_SAVING_MODE_Exec(void);
void INC_POWER_SAVING_Exec(void);
void DEC_POWER_SAVING_Exec(void);

void DxS_SETTING_Exec(void);
void DxS_INC_Exec(void);
void DxS_DEC_Exec(void);

void KEYS_SUMMARY_DISPLAY_Exec(void);

void KEY0_SIGNAL_REFERENCE_Exec(void);
void KEY1_SIGNAL_REFERENCE_Exec(void);
void KEY2_SIGNAL_REFERENCE_Exec(void);
void KEY3_SIGNAL_REFERENCE_Exec(void);
void KEY4_SIGNAL_REFERENCE_Exec(void);
void KEY5_SIGNAL_REFERENCE_Exec(void);
void KEY6_SIGNAL_REFERENCE_Exec(void);
void KEY7_SIGNAL_REFERENCE_Exec(void);
void KEY8_SIGNAL_REFERENCE_Exec(void);
void KEY9_SIGNAL_REFERENCE_Exec(void);

void KEY9_DETECT_TH_Exec(void);
void KEY9_DETECT_TH_CHANGE_Exec(void);
void KEY9_DETECT_TH_INC_Exec(void);
void KEY9_DETECT_TH_DEC_Exec(void);

void KEY9_ENDDETECT_TH_Exec(void);
void KEY9_ENDDETECT_TH_CHANGE_Exec(void);
void KEY9_ENDDETECT_TH_INC_Exec(void);
void KEY9_ENDDETECT_TH_DEC_Exec(void);

void KEY9_RECALIB_TH_Exec(void);
void KEY9_RECALIB_TH_CHANGE_Exec(void);
void KEY9_RECALIB_TH_INC_Exec(void);
void KEY9_RECALIB_TH_DEC_Exec(void);

void Slider_STATE_Exec(void);
void Slider_SIGNAL_REFERENCE_Exec(void);
void Slider_SIGNAL_POSITION_Exec(void);
void Slider_RESOLUTION_DIRECTION_Exec(void);
void Slider_RESOLUTION_CHANGE_Exec(void);
void Slider_DEC_SW_RESOLUTION_Exec(void);
void Slider_INC_SW_RESOLUTION_Exec(void);

void Wheel_STATE_Exec(void);
void Wheel_SIGNAL_REFERENCE_Exec(void);
void Wheel_SIGNAL_POSITION_Exec(void);
void Wheel_RESOLUTION_DIRECTION_Exec(void);
void Wheel_RESOLUTION_CHANGE_Exec(void);
void Wheel_DEC_SW_RESOLUTION_Exec(void);
void Wheel_INC_SW_RESOLUTION_Exec(void);

void Display_KeyState(void);
void Display_Key_Signal_Reference(void);
void Display_Action(void);
void LowPower_Action(void);
void KeyPad_Action(void);
void LED_Toggle(void);
void SystemRecovery_Action(void);

#endif /* __DEMO_H */

 // (C) COPYRIGHT 2010 STMicroelectronics *****END OF FILE***
