/**
  ********************************************************************************
  * @file    joystick_button.h
  * @brief   This file contains functions prototypes for the joystick button.
  * @author  STMicroelectronics - MCD Application Team
  * @version V1.2.0
  * @date    04-NOV-2010
  ******************************************************************************
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2010 STMicroelectronics</center></h2>
  * @image html logo.bmp
  ******************************************************************************
  */

// Define to prevent recursive inclusion -------------------------------------
#ifndef __JOYSTICK_BUTTON_H
#define __JOYSTICK_BUTTON_H

// Includes ------------------------------------------------------------------
#include "stm8l15x.h"

// Exported types ------------------------------------------------------------
/** Joystick ID
  * @note Table element must not be equal to 0 as this element is
  * used to detect the menu end of table.
 */
typedef enum {
  J_NONE = 1,
  J_RELEASED,
  UP,
  RIGHT,
  LEFT,
  DOWN,
  SEL
} JOYSTICK_ID_T;

 // Button ID
typedef enum {
  B_NONE = 1,
  B_RELEASED,
  B_PRESSED
} BUTTON_ID_T;

// Exported constants --------------------------------------------------------
// Button Status - bit definition 
#define BUTTON_STATUS_JOYSTICK_DEBOUNCE_ENABLE    0
#define BUTTON_STATUS_BUTTON_DEBOUNCE_ENABLE      1

// Exported macro ------------------------------------------------------------
// Exported variables --------------------------------------------------------
extern u8 ButtonStatus;
extern u8 ForceUpdateDisplay;

// Exported functions --------------------------------------------------------
JOYSTICK_ID_T ReadJoystick(void);
BUTTON_ID_T ReadButton(void);

#endif /*__JOYSTICK_BUTTON_H */

 // (C) COPYRIGHT 2010 STMicroelectronics *****END OF FILE***

