/**
  ********************************************************************************
  * @file    menu.h
  * @brief   This file provides the menu function prototype.
  * @author  STMicroelectronics - MCD Application Team
  * @version V1.2.0
  * @date    04-NOV-2010
  ******************************************************************************
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2010 STMicroelectronics</center></h2>
  * @image html logo.bmp
  ******************************************************************************
  */

// Define to prevent recursive inclusion -------------------------------------
#ifndef __MENU_H
#define __MENU_H

// Includes ------------------------------------------------------------------
#include "joystick_button.h"
#include "mono_lcd.h"
#include "demo.h"

// Exported define ------------------------------------------------------------

#define MENU_TOP_TEXT                 ("  ** MENU **   ")
#define HELP_TEXT                     ("     Help      ")
#define KEY_INFO_TEXT                 (" Channels Info ")
#define DETECTION_TIMEOUT_TEXT        ("Detect Timeout ")
#define DEBOUNCE_FILTER_TEXT          ("Debounce Filter")
#define POWER_SAVING_MODE_TEXT        ("Low Power Mode ")
#define DxS_SETTING_TEXT              ("      DxS      ")
#define ABOUT_TEXT                    ("     About     ")

#define KEY0KEY9_SELECTION_TEXT       (" Select K0-K9  ")
#define Slider_SELECTION_TEXT         (" Select Slider ")
#define Wheel_SELECTION_TEXT          (" Select Wheel  ")

#define Slider_STATE_TEXT             (" Slider State  ")
#define Wheel_STATE_TEXT              ("  Wheel State  ")

// Following delay is based on 5ms time base 
// Define how long each each help message is displayed - Delay = 2s 
#define HELP_PERIOD ((u16)400)

#define HELP_LINE1_TEXT (" LEFT or RIGHT\nto scroll menu ")
#define HELP_LINE2_TEXT ("   DOWN to    \nenter sub-menu ")
#define HELP_LINE3_TEXT ("     UP to    \n exit sub-menu ")

/** State ID
  * @note Table element must not be equal to 0 as this element is
  * used to detect the end of table.
 */
typedef enum {
  
  // MENU Level 1 
  
  INIT = 0, // Warning: INIT state must always be equal to ZERO !!!
  HELP = 1,
  KEY_INFO,
  DETECTION_TIMEOUT,
  DEBOUNCE_FILTER,
  POWER_SAVING_MODE,
  DxS_SETTING,
  ABOUT,
  
  // MENU Level 2 
  
  EXE_KEYS_SUMMARY_DISPLAY,
  KEY0KEY9_SELECTION,
  Slider_SELECTION,
  Wheel_SELECTION,
  
  // MENU Level 3 
  
  KEY0_SIGNAL_REFERENCE,
  KEY1_SIGNAL_REFERENCE,
  KEY2_SIGNAL_REFERENCE,
  KEY3_SIGNAL_REFERENCE,
  KEY4_SIGNAL_REFERENCE,
  KEY5_SIGNAL_REFERENCE,
  KEY6_SIGNAL_REFERENCE,
  KEY7_SIGNAL_REFERENCE,
  KEY8_SIGNAL_REFERENCE,
  KEY9_SIGNAL_REFERENCE,

  Slider_STATE,
  Slider_SIGNAL_REFERENCE,
  Slider_SIGNAL_POSITION,
  Slider_RESOLUTION_DIRECTION,
  Slider_RESOLUTION_CHANGE,
  
  Wheel_STATE,
  Wheel_SIGNAL_REFERENCE,
  Wheel_SIGNAL_POSITION,
  Wheel_RESOLUTION_DIRECTION,
  Wheel_RESOLUTION_CHANGE,

  KEY9_DETECT_TH,
  KEY9_DETECT_TH_CHANGE,
  KEY9_ENDDETECT_TH,
  KEY9_ENDDETECT_TH_CHANGE,
  KEY9_RECALIB_TH,
  KEY9_RECALIB_TH_CHANGE,
  
  // Menu Execution 

  EXE_HELP,  
  EXE_DETECTION_TIMEOUT_DISPLAY,
  EXE_DEBOUNCE_FILTER_DISPLAY,
  EXE_POWER_SAVING_MODE_DISPLAY,
  EXE_DEC_DETECTION_TIMEOUT,
  EXE_INC_DETECTION_TIMEOUT,
  EXE_DEC_DEBOUNCE_FILTER,
  EXE_INC_DEBOUNCE_FILTER,
  EXE_INC_POWER_SAVING_MODE,
  EXE_DEC_POWER_SAVING_MODE,  
  EXE_DxS_SETTING_DISPLAY,
  EXE_DEC_DxS,
  EXE_INC_DxS,
  EXE_ABOUT,

  EXE_KEY9_INC_DETECT_TH,
  EXE_KEY9_DEC_DETECT_TH,
  EXE_KEY9_INC_ENDDETECT_TH,
  EXE_KEY9_DEC_ENDDETECT_TH,
  EXE_KEY9_INC_RECALIB_TH,
  EXE_KEY9_DEC_RECALIB_TH,
  
  EXE_Slider_INC_SW_RESOLUTION,
  EXE_Slider_DEC_SW_RESOLUTION,  
  EXE_Wheel_INC_SW_RESOLUTION,
  EXE_Wheel_DEC_SW_RESOLUTION
  
} STATE_ID_T;

 // Menu NextState 
typedef struct
{
  STATE_ID_T State;
  JOYSTICK_ID_T Key;
  STATE_ID_T NextState;
}
MENU_NEXTSTATE_T;

 // Menu state 
typedef struct
{
  STATE_ID_T State;
  u8* pText;
  void (*pFunc)(void);
}
MENU_STATE_T;

// Exported constants --------------------------------------------------------
// Exported macro ------------------------------------------------------------
// Exported variables --------------------------------------------------------
extern STATE_ID_T State;

// Exported functions --------------------------------------------------------
void MenuInit(void);
void MenuUpdate(JOYSTICK_ID_T ujoystick);
void MenuDisplay(void);
u8 MenuDisplayWelcomeMessage(void);
void ABOUT_Exec(void);
void HELP_Exec(void);

#endif /* __MENU_H */

 // (C) COPYRIGHT 2010 STMicroelectronics *****END OF FILE***