/**
  ******************************************************************************
  * @file    demo.c
  * @brief   This file contains all the functions used in the menus.
  * @author  STMicroelectronics - MCD Application Team
  * @version V1.2.0
  * @date    04-NOV-2010
  ******************************************************************************
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2010 STMicroelectronics</center></h2>
  * @image html logo.bmp
  ******************************************************************************
  */

// Includes ------------------------------------------------------------------
#include "demo.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macro -------------------------------------------------------------

// Private variables ---------------------------------------------------------
u8 KeyTarget;
u8 LP_Delay_Multiplicator = 2u;
u8 DxSSetting = 0; 
u16 Percent_Calculation;
u8 Nb_full_block_old = 3u;
u8 Nb_bar_old = 3u;
u8 Claps = 0u;
u8 BeeperDelay;

// Single-channel key structure to physical key mapping (Warning also defined in demo.h 
#if defined(BOARD_STMT_8L_EV1)
const u8 KEY_look_table[10] =
  {
    5, 0, 9, 4, 8, 3, 7, 2, 6, 1		
  };
#elif defined(BOARD_STM8L151CxT_TS1)
const u8 KEY_look_table[10] =
  {
    5, 7, 1, 0, 6, 9, 2, 4, 8, 3		
  };
#endif

// Private variables ---------------------------------------------------------
// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

//==============================================================================
//                           L O W   P O W E R   M O D E
//==============================================================================

/**
  ******************************************************************************
  * @brief LP Mode display and setting menu
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * RTC_SetWakeUpCounter
  * RTC_WakeUpCmd
  ******************************************************************************
  */
void POWER_SAVING_MODE_Exec(void)
{
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, POWER_SAVING_MODE_TEXT);

  LCD_ClearLine(LCD_LINE2);

  if (LP_Delay_Multiplicator > LP_DELAY_FACTOR_MIN)
  {
    LCD_SetCursorPos(LCD_LINE2, 2);
    LCD_PrintDec3((u16) (0x01u << ((u8)(LP_Delay_Multiplicator - 3u))));
    LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " ms");
		RTC_SetWakeUpCounter((u16) (0x10u << ((u8)(LP_Delay_Multiplicator - 3u))));
  }
  else
  {
    LCD_SetCursorPos(LCD_LINE2, 0);
    LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "      OFF      ");
		RTC_WakeUpCmd(DISABLE);
  }
}

/**
  ******************************************************************************
  * @brief LP Mode increment value
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * POWER_SAVING_MODE_Exec
  ******************************************************************************
  */
void INC_POWER_SAVING_Exec(void)
{
  sim();
  if (LP_Delay_Multiplicator < LP_DELAY_FACTOR_MAX)
  {
    LP_Delay_Multiplicator++;
  }
  else
  {
    LP_Delay_Multiplicator = LP_DELAY_FACTOR_MIN;
  }
  rim();
  POWER_SAVING_MODE_Exec();
}

/**
  ******************************************************************************
  * @brief LP Mode decrement value
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * POWER_SAVING_MODE_Exec
  ******************************************************************************
  */
void DEC_POWER_SAVING_Exec(void)
{
  sim();
  if (LP_Delay_Multiplicator > LP_DELAY_FACTOR_MIN)
  {
    LP_Delay_Multiplicator--;
  }
  else
  {
    LP_Delay_Multiplicator = LP_DELAY_FACTOR_MAX;
  }  
  rim();
  POWER_SAVING_MODE_Exec();
}

//==============================================================================
//                                       D X S
//==============================================================================

/**
  ******************************************************************************
  * @brief DxS display and activation menu
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void DxS_SETTING_Exec(void)
{

  // Line 1
  LCD_ClearLine(LCD_LINE1);
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, DxS_SETTING_TEXT);

  // Line 2
  LCD_ClearLine(LCD_LINE2);
  LCD_SetCursorPos(LCD_LINE2, 0);
  
  // DxS on all keys, wheel and slider
  if (DxSSetting == 1)
  {
#if NUMBER_OF_SINGLE_CHANNEL_KEYS > 0
    for (KeyIndex = 0; KeyIndex < NUMBER_OF_SINGLE_CHANNEL_KEYS; KeyIndex++)
    {
      sSCKeyInfo[KeyIndex].DxSGroup = 0x01;
    }
#endif
#if NUMBER_OF_MULTI_CHANNEL_KEYS > 0
    for (KeyIndex = 0; KeyIndex < NUMBER_OF_MULTI_CHANNEL_KEYS; KeyIndex++)
    {
      sMCKeyInfo[KeyIndex].DxSGroup = 0x01;
    }
#endif
    LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "      ALL      ");
  }
  
  // DxS on keys only
  else if (DxSSetting == 2)
  {
#if NUMBER_OF_SINGLE_CHANNEL_KEYS > 0
    for (KeyIndex = 0; KeyIndex < NUMBER_OF_SINGLE_CHANNEL_KEYS; KeyIndex++)
    {
      sSCKeyInfo[KeyIndex].DxSGroup = 0x01;
    }
#endif
#if NUMBER_OF_MULTI_CHANNEL_KEYS > 0
    for (KeyIndex = 0; KeyIndex < NUMBER_OF_MULTI_CHANNEL_KEYS; KeyIndex++)
    {
      sMCKeyInfo[KeyIndex].DxSGroup = 0; // OFF
    }
#endif    
    LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "   KEYS ONLY   ");
  }

  // DxS on keys 0 to 8 only
  else if (DxSSetting == 3)
  {
#if NUMBER_OF_SINGLE_CHANNEL_KEYS > 0
    for (KeyIndex = 0; KeyIndex < NUMBER_OF_SINGLE_CHANNEL_KEYS; KeyIndex++)
    {
      sSCKeyInfo[KeyIndex].DxSGroup = 0x01;
    }
    sSCKeyInfo[KEY_look_table[9]].DxSGroup = 0; // OFF for KEY 9
#endif
#if NUMBER_OF_MULTI_CHANNEL_KEYS > 0
    for (KeyIndex = 0; KeyIndex < NUMBER_OF_MULTI_CHANNEL_KEYS; KeyIndex++)
    {
      sMCKeyInfo[KeyIndex].DxSGroup = 0; // OFF
    }
#endif    
    LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "  KEYS 0 TO 8  ");
  }

  // DxS disabled
  else
  {
#if NUMBER_OF_SINGLE_CHANNEL_KEYS > 0
    for (KeyIndex = 0; KeyIndex < NUMBER_OF_SINGLE_CHANNEL_KEYS; KeyIndex++)
    {
      sSCKeyInfo[KeyIndex].DxSGroup = 0; // OFF
    }
#endif
#if NUMBER_OF_MULTI_CHANNEL_KEYS > 0
    for (KeyIndex = 0; KeyIndex < NUMBER_OF_MULTI_CHANNEL_KEYS; KeyIndex++)
    {
      sMCKeyInfo[KeyIndex].DxSGroup = 0; // OFF
    }
#endif
    LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "      OFF      ");
  }
}

/**
  ******************************************************************************
  * @brief DxS setting value increment
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * DxS_SETTING_Exec
  ******************************************************************************
  */
void DxS_INC_Exec(void)
{
  if (DxSSetting < DXS_SETTING_MAX)
  {
    DxSSetting++;
  }
  else
  {
    DxSSetting = DXS_SETTING_MIN;
  }
  DxS_SETTING_Exec();
}

/**
  ******************************************************************************
  * @brief DxS setting value decrement
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * DxS_SETTING_Exec
  ******************************************************************************
  */
void DxS_DEC_Exec(void)
{
  if (DxSSetting > DXS_SETTING_MIN)
  {
    DxSSetting--;
  }
  else
  {
    DxSSetting = DXS_SETTING_MAX;
  }
  DxS_SETTING_Exec();
}

//==============================================================================
//                                       D T O
//==============================================================================

/**
  ******************************************************************************
  * @brief DETECTION TIMEOUT display and setting menu
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void DETECTION_TIMEOUT_Exec(void)
{
  
  LCD_ClearLine(LCD_LINE1);
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, DETECTION_TIMEOUT_TEXT);
  
  LCD_ClearLine(LCD_LINE2);
  if (DetectionTimeout > DTO_SETTING_MIN)
  {
    LCD_SetCursorPos(LCD_LINE2, 2);
    LCD_PrintDec2(DetectionTimeout);
    LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " sec");
  }
  else
  {
    LCD_SetCursorPos(LCD_LINE2, 0);
    LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "      OFF      ");    
  }

}

/**
  ******************************************************************************
  * @brief DETECTION TIMEOUT value decrementation
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * DETECTION_TIMEOUT_Exec
  ******************************************************************************
  */
void DEC_DETECTION_TIMEOUT_Exec(void)
{
  if (DetectionTimeout > DTO_SETTING_MIN)
  {
    DetectionTimeout--;
  }
  else
  {
    DetectionTimeout = DTO_SETTING_MAX;
  }
  DETECTION_TIMEOUT_Exec();
}

/**
  ******************************************************************************
  * @brief DETECTION TIMEOUT value incrementation
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * DETECTION_TIMEOUT_Exec
  ******************************************************************************
  */
void INC_DETECTION_TIMEOUT_Exec(void)
{
  if (DetectionTimeout < DTO_SETTING_MAX)
  {
    DetectionTimeout++;
  }
  else
  {
    DetectionTimeout = DTO_SETTING_MIN;
  }
  DETECTION_TIMEOUT_Exec();
}

//==============================================================================
//                            D E B O U N C E   F I L T E R
//==============================================================================

/**
  ******************************************************************************
  * @brief DEBOUNCE FILTER display and setting menu
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void DEBOUNCE_FILTER_Exec(void)
{
  LCD_ClearLine(LCD_LINE1);
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, DEBOUNCE_FILTER_TEXT);
  
  LCD_ClearLine(LCD_LINE2);
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_PrintDec1(DetectionIntegrator);
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " Acquisitions");
}

/**
  ******************************************************************************
  * @brief DEBOUNCE FILTER decrementation
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * DEBOUNCE_FILTER_Exec
  ******************************************************************************
  */
void DEC_DEBOUNCE_FILTER_Exec(void)
{
  if (DetectionIntegrator > DEBOUNCE_SETTING_MIN)
  {
    DetectionIntegrator--;
  }
  else
  {
    DetectionIntegrator = DEBOUNCE_SETTING_MAX;
  }
  DEBOUNCE_FILTER_Exec();
}

/**
  ******************************************************************************
  * @brief DEBOUNCE FILTER incrementation
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * DEBOUNCE_FILTER_Exec
  ******************************************************************************
  */
void INC_DEBOUNCE_FILTER_Exec(void)
{
  if (DetectionIntegrator < DEBOUNCE_SETTING_MAX)
  {
    DetectionIntegrator++;
  }
  else
  {
    DetectionIntegrator = DEBOUNCE_SETTING_MIN;
  }
  DEBOUNCE_FILTER_Exec();
}

//==============================================================================
//                                      K E Y S
//==============================================================================

/**
  ******************************************************************************
  * @brief KEY0 parameters display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * Display_Key_Signal_Reference
  ******************************************************************************
  */
void KEY0_SIGNAL_REFERENCE_Exec(void)
{
  KeyTarget = 0;
  Display_Key_Signal_Reference();
}

/**
  ******************************************************************************
  * @brief KEY1 parameters display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * Display_Key_Signal_Reference
  ******************************************************************************
  */
void KEY1_SIGNAL_REFERENCE_Exec(void)
{
  KeyTarget = 1;
  Display_Key_Signal_Reference();
}

/**
  ******************************************************************************
  * @brief KEY2 parameters display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * Display_Key_Signal_Reference
  ******************************************************************************
  */
void KEY2_SIGNAL_REFERENCE_Exec(void)
{
  KeyTarget = 2;
  Display_Key_Signal_Reference();
}

/**
  ******************************************************************************
  * @brief KEY3 parameters display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * Display_Key_Signal_Reference
  ******************************************************************************
  */
void KEY3_SIGNAL_REFERENCE_Exec(void)
{
  KeyTarget = 3;
  Display_Key_Signal_Reference();
}

/**
  ******************************************************************************
  * @brief KEY4 parameters display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * Display_Key_Signal_Reference
  ******************************************************************************
  */
void KEY4_SIGNAL_REFERENCE_Exec(void)
{
  KeyTarget = 4;
  Display_Key_Signal_Reference();
}

/**
  ******************************************************************************
  * @brief KEY5 parameters display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * Display_Key_Signal_Reference
  ******************************************************************************
  */
void KEY5_SIGNAL_REFERENCE_Exec(void)
{
  KeyTarget = 5;
  Display_Key_Signal_Reference();
}

/**
  ******************************************************************************
  * @brief KEY6 parameters display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * Display_Key_Signal_Reference
  ******************************************************************************
  */
void KEY6_SIGNAL_REFERENCE_Exec(void)
{
  KeyTarget = 6;
  Display_Key_Signal_Reference();
}

/**
  ******************************************************************************
  * @brief KEY7 parameters display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * Display_Key_Signal_Reference
  ******************************************************************************
  */
void KEY7_SIGNAL_REFERENCE_Exec(void)
{
  KeyTarget = 7;
  Display_Key_Signal_Reference();
}

/**
  ******************************************************************************
  * @brief KEY8 parameters display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * Display_Key_Signal_Reference
  ******************************************************************************
  */
void KEY8_SIGNAL_REFERENCE_Exec(void)
{
  KeyTarget = 8;
  Display_Key_Signal_Reference();
}

/**
  ******************************************************************************
  * @brief KEY9 parameters display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * Display_Key_Signal_Reference
  ******************************************************************************
  */
void KEY9_SIGNAL_REFERENCE_Exec(void)
{
  KeyTarget = 9;
  Display_Key_Signal_Reference();
}

/**
  ******************************************************************************
  * @brief KEY9 Detection Threshold display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void KEY9_DETECT_TH_Exec(void)
{
  LCD_ClearLine(LCD_LINE1);
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, " K9 Detect Thr ");
  
  LCD_ClearLine(LCD_LINE2);
  LCD_SetCursorPos(LCD_LINE2, 2);
  LCD_PrintDec3(KEY9.DetectThreshold);
}

/**
  ******************************************************************************
  * @brief KEY9 Detection Threshold setting
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void KEY9_DETECT_TH_CHANGE_Exec(void)
{
  LCD_ClearLine(LCD_LINE1);
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "<K9 Detect Thr>");
  
  LCD_ClearLine(LCD_LINE2);
  LCD_SetCursorPos(LCD_LINE2, 2);
  LCD_PrintDec3(KEY9.DetectThreshold);
}

/**
  ******************************************************************************
  * @brief KEY9 Detection Threshold incrementation
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * KEY9_DETECT_TH_CHANGE_Exec
  ******************************************************************************
  */
void KEY9_DETECT_TH_INC_Exec(void)
{
  if (KEY9.DetectThreshold < DETECT_TH_MAX)
  {
    KEY9.DetectThreshold++;
  }
  KEY9_DETECT_TH_CHANGE_Exec();
}

/**
  ******************************************************************************
  * @brief KEY9 Detection Threshold decrementation
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * KEY9_DETECT_TH_CHANGE_Exec
  ******************************************************************************
  */
void KEY9_DETECT_TH_DEC_Exec(void)
{
  if (KEY9.DetectThreshold > DETECT_TH_MIN)
  {
    if (KEY9.DetectThreshold == (KEY9.EndDetectThreshold + 1))
    {
      if (KEY9.EndDetectThreshold > END_DETECT_TH_MIN)
      {
        KEY9.EndDetectThreshold--;
      }
    }
    KEY9.DetectThreshold--;
  }
  KEY9_DETECT_TH_CHANGE_Exec();
}

/**
  ******************************************************************************
  * @brief KEY9 End Detection Threshold display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void KEY9_ENDDETECT_TH_Exec(void)
{
  LCD_ClearLine(LCD_LINE1);
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, " K9 EndDet Thr ");
  
  LCD_ClearLine(LCD_LINE2);
  LCD_SetCursorPos(LCD_LINE2, 2);
  LCD_PrintDec3(KEY9.EndDetectThreshold);
}

/**
  ******************************************************************************
  * @brief KEY9 End Detection Threshold setting
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void KEY9_ENDDETECT_TH_CHANGE_Exec(void)
{
  LCD_ClearLine(LCD_LINE1);
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "<K9 EndDet Thr>");
  
  LCD_ClearLine(LCD_LINE2);
  LCD_SetCursorPos(LCD_LINE2, 2);
  LCD_PrintDec3(KEY9.EndDetectThreshold);
}

/**
  ******************************************************************************
  * @brief KEY9 End Detection Threshold incrementation
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * KEY9_ENDDETECT_TH_CHANGE_Exec
  ******************************************************************************
  */
void KEY9_ENDDETECT_TH_INC_Exec(void)
{
  if (KEY9.EndDetectThreshold < END_DETECT_TH_MAX)
  {
    if (KEY9.EndDetectThreshold == (KEY9.DetectThreshold - 1))
    {
      if (KEY9.DetectThreshold < DETECT_TH_MAX)
      {
        KEY9.DetectThreshold++;
      }
    }    
    KEY9.EndDetectThreshold++;
  }
  KEY9_ENDDETECT_TH_CHANGE_Exec();
}

/**
  ******************************************************************************
  * @brief KEY9 End Detection Threshold decrementation
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * KEY9_ENDDETECT_TH_CHANGE_Exec
  ******************************************************************************
  */
void KEY9_ENDDETECT_TH_DEC_Exec(void)
{
  if (KEY9.EndDetectThreshold > END_DETECT_TH_MIN)
  {
    KEY9.EndDetectThreshold--;
  }
  KEY9_ENDDETECT_TH_CHANGE_Exec();
}

/**
  ******************************************************************************
  * @brief KEY9 Recalibration Threshold display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void KEY9_RECALIB_TH_Exec(void)
{
  LCD_ClearLine(LCD_LINE1);
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, " K9 Recal  Thr ");
  
  LCD_ClearLine(LCD_LINE2);
  LCD_SetCursorPos(LCD_LINE2, 2);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "-");
  LCD_PrintDec3(-(KEY9.RecalibrationThreshold));
}

/**
  ******************************************************************************
  * @brief KEY9 Recalibration Threshold setting
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void KEY9_RECALIB_TH_CHANGE_Exec(void)
{
  LCD_ClearLine(LCD_LINE1);
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "<K9 Recal  Thr>");
  
  LCD_ClearLine(LCD_LINE2);
  LCD_SetCursorPos(LCD_LINE2, 2);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "-");
  LCD_PrintDec3(-(KEY9.RecalibrationThreshold));
}

/**
  ******************************************************************************
  * @brief KEY9 Recalibration Threshold incrementation
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * KEY9_RECALIB_TH_CHANGE_Exec
  ******************************************************************************
  */
void KEY9_RECALIB_TH_INC_Exec(void)
{
  u8 tmpval;
  tmpval = -KEY9.RecalibrationThreshold;
  if (tmpval < RECALIB_TH_MAX)
  {
    tmpval++;
  }
  KEY9.RecalibrationThreshold = -tmpval;
  KEY9_RECALIB_TH_CHANGE_Exec();
}

/**
  ******************************************************************************
  * @brief KEY9 Recalibration Threshold decrementation
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * KEY9_RECALIB_TH_CHANGE_Exec
  ******************************************************************************
  */
void KEY9_RECALIB_TH_DEC_Exec(void)
{
  u8 tmpval;
  tmpval = -KEY9.RecalibrationThreshold;
  if (tmpval > RECALIB_TH_MIN)
  {
    tmpval--;
  }
  KEY9.RecalibrationThreshold = -tmpval;
  KEY9_RECALIB_TH_CHANGE_Exec();
}

/**
  ******************************************************************************
  * @brief KEY0 to KEY9 parameters display (Reference, Signal, Delta and State)
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void Display_Key_Signal_Reference(void)
{
  s16 delta;
  
  delta = sSCKeyInfo[KEY_look_table[KeyTarget]].Channel.Reference - sSCKeyInfo[KEY_look_table[KeyTarget]].Channel.LastMeas;
  if (delta < 0) // Noise
  {
    delta = 0;
  }
  
  // Line 1: Reference and Signal
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "K");
  LCD_PrintDec1(KeyTarget);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "  R");
  LCD_PrintDec4(sSCKeyInfo[KEY_look_table[KeyTarget]].Channel.Reference);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, " S");
  LCD_PrintDec4(sSCKeyInfo[KEY_look_table[KeyTarget]].Channel.LastMeas);  
  
  // Line 2: State and Delta
  LCD_SetCursorPos(LCD_LINE2, 0);
  switch (sSCKeyInfo[KEY_look_table[KeyTarget]].State.whole)
  {
    case PRE_CALIBRATION_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "PRECALIB  ");
      break;
    case CALIBRATION_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "CALIB     ");
      break;
    case IDLE_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "IDLE      ");
      break;
    case DETECTED_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "DETECTED  ");
      break;
    case ERROR_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "ERROR     ");
      break;
    case PRE_DETECTED_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "PREDETECT ");
      break;
    case POST_DETECTED_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "POSTDETECT");
      break;
    case DISABLED_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "DISABLED  ");
      break;
    default:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "UNKN STATE");
      break;
  }  
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " D");
  LCD_PrintDec3((u16)delta);
}

/**
  ******************************************************************************
  * @brief KEYs, Wheel and Slider touched status display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void KEYS_SUMMARY_DISPLAY_Exec(void)
{

  // Line 1: Keys 0 to 4 and Wheel position
  LCD_SetCursorPos(LCD_LINE1, 0);

  for ( KeyIndex = 0; KeyIndex < (NUMBER_OF_SINGLE_CHANNEL_KEYS / 2); KeyIndex++ )
  {
    
    LCD_PrintDec1((u8) (KeyIndex));
   
    // Check if any key is in DETECT state 
    if (sSCKeyInfo[KEY_look_table[KeyIndex]].Setting.b.DETECTED)
    {
      LCD_PrintChar(0x11u);
    }
    else
    {
      LCD_PrintChar(0x20u);
    }
  }

  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "W:");
  LCD_PrintHex2(sMCKeyInfo[0].Position);

  if (sMCKeyInfo[0].Setting.b.DETECTED)
  {
    LCD_PrintChar(0x11u);
  }
  else
  {
    LCD_PrintChar(0x20u);
  }

  // Line 2: Keys 5 to 9 and Slider position
  LCD_SetCursorPos(LCD_LINE2, 0);

  for ( KeyIndex = NUMBER_OF_SINGLE_CHANNEL_KEYS / 2; KeyIndex < NUMBER_OF_SINGLE_CHANNEL_KEYS; KeyIndex++ )
  {
    
    LCD_PrintDec1((u8) (KeyIndex));
   
    // Check if any key is in DETECT state 
    if (sSCKeyInfo[KEY_look_table[KeyIndex]].Setting.b.DETECTED)
    {
      LCD_PrintChar(0x11u);
    }
    else
    {
      LCD_PrintChar(0x20u);
    }
  }

  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "S:");
  LCD_PrintHex2(sMCKeyInfo[1].Position);

  if (sMCKeyInfo[1].Setting.b.DETECTED)
  {
    LCD_PrintChar(0x11u);
  }
  else
  {
    LCD_PrintChar(0x20u);
  }

}

//==============================================================================
//                                  S L I D E R
//==============================================================================

/**
  ******************************************************************************
  * @brief Slider state display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void Slider_STATE_Exec(void)
{
  
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, Slider_STATE_TEXT);
  
  LCD_SetCursorPos(LCD_LINE2, 0);

  switch (sMCKeyInfo[1].State.whole)
  {
    case PRE_CALIBRATION_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "PRE CALIBRATION");
      break;
    case CALIBRATION_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "  CALIBRATION  ");
      break;
    case IDLE_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "     IDLE      ");
      break;
    case DETECTED_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "    DETECTED   ");
      break;
    case ERROR_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "     ERROR     ");
      break;
    case PRE_DETECTED_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "  PRE DETECTED ");
      break;
    case POST_DETECTED_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " POST DETECTED ");
      break;
    case DISABLED_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "    DISABLED   ");
      break;
    default:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " UNKNOWN STATE ");
      break;
  }
}

/**
  ******************************************************************************
  * @brief Slider Reference and Signal display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void Slider_SIGNAL_REFERENCE_Exec(void)
{
  // Line 1: Reference
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "R");
  LCD_PrintDec4(sMCKeyInfo[1].Channel[0].Reference);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, " ");
  LCD_PrintDec4(sMCKeyInfo[1].Channel[1].Reference);  
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, " ");
  LCD_PrintDec4(sMCKeyInfo[1].Channel[2].Reference);  
  
  // Line 2: Signal (Last Measure)
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "S");
  LCD_PrintDec4(sMCKeyInfo[1].Channel[0].LastMeas);
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " ");
  LCD_PrintDec4(sMCKeyInfo[1].Channel[1].LastMeas);  
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " ");
  LCD_PrintDec4(sMCKeyInfo[1].Channel[2].LastMeas);    
}

/**
  ******************************************************************************
  * @brief Slider Delta and Position display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void Slider_SIGNAL_POSITION_Exec(void)
{
  s16 delta;
  u8 i;
  
  // Line 1: Delta
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "D ");
  
  for (i = 0; i < 3; i++)
  {
    delta = sMCKeyInfo[1].Channel[i].Reference - sMCKeyInfo[1].Channel[i].LastMeas;
    if (delta < 0) // Noise
    {
      delta = 0;
    }
    LCD_PrintDec3((u16)delta);
    LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, " ");
  }
  
  // Line 2: Position
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "P=0x");
  LCD_PrintHex2(sMCKeyInfo[1].UnScaledPosition);
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "  FP=0x");
  LCD_PrintHex2(sMCKeyInfo[1].Position);
}

/**
  ******************************************************************************
  * @brief Slider Resolution display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void Slider_RESOLUTION_DIRECTION_Exec(void)
{
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "  S Resolution "); 
  LCD_ClearLine(LCD_LINE2);
	LCD_SetCursorPos(LCD_LINE2, 2);
  LCD_PrintDec1(sMCKeyInfo[1].Resolution);
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " bits");
}

/**
  ******************************************************************************
  * @brief Slider Resolution setting
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void Slider_RESOLUTION_CHANGE_Exec(void)
{
  LCD_SetCursorPos(LCD_LINE1, 0); 
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "< S Resolution>");
	LCD_SetCursorPos(LCD_LINE2, 2);
  LCD_PrintDec1(sMCKeyInfo[1].Resolution);
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " bits");
}

/**
  ******************************************************************************
  * @brief Slider Resolution incrementation
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * Slider_RESOLUTION_CHANGE_Exec
  ******************************************************************************
  */
void Slider_INC_SW_RESOLUTION_Exec(void)
{
  if (sMCKeyInfo[1].Resolution < SLIDER_WHEEL_RESOLUTION_MAX)
  {
    sMCKeyInfo[1].Resolution++;
  }
  else
  {
    sMCKeyInfo[1].Resolution = SLIDER_WHEEL_RESOLUTION_MIN;
  }
  Slider_RESOLUTION_CHANGE_Exec();
}

/**
  ******************************************************************************
  * @brief Slider Resolution decrementation
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * Slider_RESOLUTION_CHANGE_Exec
  ******************************************************************************
  */
void Slider_DEC_SW_RESOLUTION_Exec(void)
{
  if (sMCKeyInfo[1].Resolution > SLIDER_WHEEL_RESOLUTION_MIN)
  {
    sMCKeyInfo[1].Resolution--;
  }
  else
  {
    sMCKeyInfo[1].Resolution = SLIDER_WHEEL_RESOLUTION_MAX;
  }
  Slider_RESOLUTION_CHANGE_Exec();
}

//==============================================================================
//                                   W H E E L
//==============================================================================

/**
  ******************************************************************************
  * @brief Wheel state display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void Wheel_STATE_Exec(void)
{
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, Wheel_STATE_TEXT);
  
  LCD_SetCursorPos(LCD_LINE2, 0);

  switch (sMCKeyInfo[0].State.whole)
  {
    case PRE_CALIBRATION_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "PRE CALIBRATION");
      break;
    case CALIBRATION_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "  CALIBRATION  ");
      break;
    case IDLE_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "     IDLE      ");
      break;
    case DETECTED_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "    DETECTED   ");
      break;
    case ERROR_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "     ERROR     ");
      break;
    case PRE_DETECTED_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "  PRE DETECTED ");
      break;
    case POST_DETECTED_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " POST DETECTED ");
      break;
    case DISABLED_STATE:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "    DISABLED   ");
      break;
    default:
      LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " UNKNOWN STATE ");
      break;
  }
}

/**
  ******************************************************************************
  * @brief Wheel Reference and Signal display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void Wheel_SIGNAL_REFERENCE_Exec(void)
{
  // Line 1: Reference
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "R");
  LCD_PrintDec4(sMCKeyInfo[0].Channel[0].Reference);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, " ");
  LCD_PrintDec4(sMCKeyInfo[0].Channel[1].Reference);  
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, " ");
  LCD_PrintDec4(sMCKeyInfo[0].Channel[2].Reference);  
  
  // Line 2: Signal (Last measure) 
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "S");
  LCD_PrintDec4(sMCKeyInfo[0].Channel[0].LastMeas);
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " ");
  LCD_PrintDec4(sMCKeyInfo[0].Channel[1].LastMeas);  
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " ");
  LCD_PrintDec4(sMCKeyInfo[0].Channel[2].LastMeas);  
}

/**
  ******************************************************************************
  * @brief Wheel Delta and Position display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void Wheel_SIGNAL_POSITION_Exec(void)
{
  s16 delta;
  u8 i;
  
  // Line 1: Delta
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "D ");
  
  for (i = 0; i < 3; i++)
  {
    delta = sMCKeyInfo[0].Channel[i].Reference - sMCKeyInfo[0].Channel[i].LastMeas;
    if (delta < 0) // Noise
    {
      delta = 0;
    }
    LCD_PrintDec3((u16)delta);
    LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, " ");
  }
  
  // Line 2: Position
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "P=0x");
  LCD_PrintHex2(sMCKeyInfo[0].UnScaledPosition);
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "  FP=0x");
  LCD_PrintHex2(sMCKeyInfo[0].Position);
}

/**
  ******************************************************************************
  * @brief Wheel Resolution display
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void Wheel_RESOLUTION_DIRECTION_Exec(void)
{
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "  W Resolution ");
	LCD_ClearLine(LCD_LINE2);
	LCD_SetCursorPos(LCD_LINE2, 2);
  LCD_PrintDec1(sMCKeyInfo[0].Resolution);
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " bits");
}

/**
  ******************************************************************************
  * @brief Wheel Resolution setting
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void Wheel_RESOLUTION_CHANGE_Exec(void)
{
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_PrintString(LCD_LINE1, DISABLE, ENABLE, "< W Resolution>");
  LCD_SetCursorPos(LCD_LINE2, 2);
  LCD_PrintDec1(sMCKeyInfo[0].Resolution);
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, " bits");
}

/**
  ******************************************************************************
  * @brief Wheel Resolution incrementation
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * Wheel_RESOLUTION_CHANGE_Exec
  ******************************************************************************
  */
void Wheel_INC_SW_RESOLUTION_Exec(void)
{
  if (sMCKeyInfo[0].Resolution < SLIDER_WHEEL_RESOLUTION_MAX)
  {
    sMCKeyInfo[0].Resolution++;
  }
  else
  {
    sMCKeyInfo[0].Resolution = SLIDER_WHEEL_RESOLUTION_MIN;
  }
  Wheel_RESOLUTION_CHANGE_Exec();
}

/**
  ******************************************************************************
  * @brief Wheel Resolution decrementation
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * Wheel_RESOLUTION_CHANGE_Exec
  ******************************************************************************
  */
void Wheel_DEC_SW_RESOLUTION_Exec(void)
{
  if (sMCKeyInfo[0].Resolution > SLIDER_WHEEL_RESOLUTION_MIN)
  {
    sMCKeyInfo[0].Resolution--;
  }
  else
  {
    sMCKeyInfo[0].Resolution = SLIDER_WHEEL_RESOLUTION_MAX;
  }
  Wheel_RESOLUTION_CHANGE_Exec();
}

//==============================================================================
//                                   M I S C.
//==============================================================================

/**
 ******************************************************************************
 * @brief Manages the LCD display according to menu state:
 * @par Parameters:
 * None
 * @retval void None
 * @par Required preconditions:
 * None
 * @par Called functions:
 * KEYS_SUMMARY_DISPLAY_Exec
 * Display_Key_Signal_Reference
 * Slider_STATE_Exec
 * Slider_SIGNAL_REFERENCE_Exec
 * Slider_SIGNAL_POSITION_Exec
 * Wheel_STATE_Exec
 * Wheel_SIGNAL_REFERENCE_Exec
 * Wheel_SIGNAL_POSITION_Exec 
 ******************************************************************************
 */
void Display_Action(void)
{
  if (TSLState == TSL_IDLE_STATE)
  {
    switch (State)
    {
      
      case EXE_KEYS_SUMMARY_DISPLAY:
        if ((TSL_GlobalSetting.b.CHANGED) || (TSL_GlobalSetting.b.POSCHANGED))
        {
          KEYS_SUMMARY_DISPLAY_Exec();
          sMCKeyInfo[0].Setting.b.POSCHANGED = 0;
          sMCKeyInfo[1].Setting.b.POSCHANGED = 0;          
        }
        break;

      case KEY0_SIGNAL_REFERENCE:
      case KEY1_SIGNAL_REFERENCE:
      case KEY2_SIGNAL_REFERENCE:
      case KEY3_SIGNAL_REFERENCE:
      case KEY4_SIGNAL_REFERENCE:
      case KEY5_SIGNAL_REFERENCE:
      case KEY6_SIGNAL_REFERENCE:
      case KEY7_SIGNAL_REFERENCE:
      case KEY8_SIGNAL_REFERENCE:
      case KEY9_SIGNAL_REFERENCE:
        Display_Key_Signal_Reference();
        break;

      case Slider_STATE:
        Slider_STATE_Exec();
        break;

      case Slider_SIGNAL_REFERENCE:
        Slider_SIGNAL_REFERENCE_Exec();
        break;
        
      case Slider_SIGNAL_POSITION:
        Slider_SIGNAL_POSITION_Exec();
        break;

      case Wheel_STATE:
        Wheel_STATE_Exec();
        break;

      case Wheel_SIGNAL_REFERENCE:
        Wheel_SIGNAL_REFERENCE_Exec();
        break;
        
      case Wheel_SIGNAL_POSITION:
        Wheel_SIGNAL_POSITION_Exec();
        break;

      default:
        break;
    }
  }
}

/**
 ******************************************************************************
 * @Update key state to the user application and generate beep
 * @par Parameters:
 * None
 * @retval void None
 * @par Required preconditions:
 * None
 * @par Called functions:
 * PlayPWM
 ******************************************************************************
 */
void KeyPad_Action(void)
{
  u8 TotalDetected; // Count the number of keys detected
  u8 KeyIndex;
  static u8 prev_TotalDetected = 0;

  if (TSL_GlobalSetting.b.CHANGED)
  {
    if (TSL_GlobalSetting.b.DETECTED)
    {
      
      TotalDetected = 0;
     
#if NUMBER_OF_SINGLE_CHANNEL_KEYS > 0
    for (KeyIndex = 0; KeyIndex < NUMBER_OF_SINGLE_CHANNEL_KEYS; KeyIndex++)
    {
      if (sSCKeyInfo[KeyIndex].Setting.b.DETECTED)
      {
        TotalDetected++;
      }
    }
#endif
   
#if NUMBER_OF_MULTI_CHANNEL_KEYS > 0
      for (KeyIndex = 0; KeyIndex < NUMBER_OF_MULTI_CHANNEL_KEYS; KeyIndex++)
      {
        if (sMCKeyInfo[KeyIndex].Setting.b.DETECTED)
        {
          TotalDetected++;
        }
      }
#endif      
     
      // Bip only if the number of touches increases
      if (TotalDetected > prev_TotalDetected)
      {
        BeeperDelay = 10;
        PlayPWM(1000u);
      }
      
      prev_TotalDetected = TotalDetected;
      
    }
    else
    {
      prev_TotalDetected = 0;
    }
    
    TSL_GlobalSetting.b.CHANGED = 0;
  }
}

/**
 ******************************************************************************
  * @brief HALTs the micro if:
	* - TSL State machine is in TSL_IDLE_STATE
	* - Low Power mode ENABLED
	* - No key is in DETECT
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * RTC_WakeUpCmd
 ******************************************************************************
 */
void LowPower_Action(void)
{
  if (TSLState == TSL_IDLE_STATE)
  {
    if ((LP_Delay_Multiplicator > 2u) && !((TSL_GlobalState.b.DETECTED)||(TSL_GlobalState.b.PRE_STATE)||(TSL_GlobalState.b.POST_STATE)||(TSL_GlobalState.b.CALIBRATION)))
    {
			RTC_WakeUpCmd(ENABLE);
      halt();
    }
  }
}

/**
 ******************************************************************************
 * @Toggle LED of the user board
 * @par Parameters:
 * None
 * @retval void None
 * @par Required preconditions:
 * None
 * @par Called functions:
 * None
 ******************************************************************************
 */
void LED_Toggle(void)
{
  static u8 LED_Mask = 0x01;
  static u8 LEDDelay = 0u;

  if (LEDDelay < 50u)
  {
    LEDDelay++;
  }
  else
  {
    if (LED_Mask == 0x02u)
    {
      LED_Mask = 0x01;
    }
    else
    {
      LED_Mask = (u8) (LED_Mask << 1u);
    }
    LEDDelay = 0;
  }

  switch (LED_Mask)
  {
    case 0x01:
      LED2_PORT_ODR |= LED2_PIN_MASK;			
      break;
   
    case 0x02:
      LED2_PORT_ODR &= (~LED2_PIN_MASK);
      break;
   
    default:
      break;
  }
}

/**
 ******************************************************************************
 * @Check if key, wheel or slider error, then launch calibration
 * @par Parameters:
 * None
 * @retval void None
 * @par Required preconditions:
 * None
 * @par Called functions:
 * None
 ******************************************************************************
 */
void SystemRecovery_Action(void)
{
  u8 i;
  static u8 SCKey_Recovery_Counter[NUMBER_OF_SINGLE_CHANNEL_KEYS];
  static u8 MCKey_Recovery_Counter[NUMBER_OF_MULTI_CHANNEL_KEYS];
	
  // For all keys and slider launch a calibration by disabling and enabling the element in error  
#if NUMBER_OF_SINGLE_CHANNEL_KEYS > 0
  for (i = 0; i < NUMBER_OF_SINGLE_CHANNEL_KEYS; i++)
  {
		if (SCKey_Recovery_Counter[i] < 10) /* try 9 times to recover */
		{
			if (sSCKeyInfo[i].Setting.b.ENABLED == 0)
      {
        sSCKeyInfo[i].Setting.b.ENABLED = 1;
      }
      if (sSCKeyInfo[i].Setting.b.ERROR == 1)
      {
        sSCKeyInfo[i].Setting.b.ENABLED = 0;
				SCKey_Recovery_Counter[i]++;
      }
		}
		if ((SCKey_Recovery_Counter[i]) && (sSCKeyInfo[i].State.b.IDLE)) /* reset counter if system is recovered */
		{
			SCKey_Recovery_Counter[i] = 0;
	  }
  }
#endif

#if NUMBER_OF_MULTI_CHANNEL_KEYS > 0
  for (i = 0; i < NUMBER_OF_MULTI_CHANNEL_KEYS; i++)
  {
		if (MCKey_Recovery_Counter[i] < 10) /* try 9 times to recover */
		{
      if (sMCKeyInfo[i].Setting.b.ERROR == 1)
      {
        sMCKeyInfo[i].Setting.b.ENABLED = 0;
      }
      if (sMCKeyInfo[i].Setting.b.ENABLED == 0)
      {
      sMCKeyInfo[i].Setting.b.ENABLED = 1;
			MCKey_Recovery_Counter[i]++;
      }
	  }
		if ((MCKey_Recovery_Counter[i]) && (sMCKeyInfo[i].State.b.IDLE)) /* reset counter if system is recovered */
		{
			MCKey_Recovery_Counter[i] = 0;
	  }
  }
#endif
}

// Public functions ----------------------------------------------------------

 // (C) COPYRIGHT 2010 STMicroelectronics *****END OF FILE***
