/**
  ******************************************************************************
  * @file    main.c
  * @brief   This file contains the main function.
  * @author  STMicroelectronics - MCD Application Team
  * @version V1.2.0
  * @date    04-NOV-2010
  ******************************************************************************
  */

// Includes ------------------------------------------------------------------
#include "main.h"
#include "stm8_tsl_api.h"
#include "joystick_button.h"
#include "functions.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------
// Public functions ----------------------------------------------------------

/**
  ******************************************************************************
  * @brief Validation firmware main entry point.
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Library functions called:
  ******************************************************************************
  */
void main(void)
{
  
  JOYSTICK_ID_T Key;
  u8 IntroSkipped;
  
  // Init the Application clock and peripherals, menu 
  Init_Application();

  // Display the welcome text 
	IntroSkipped = MenuDisplayWelcomeMessage();

  // Init Touch sensing library 
  TSL_Init();

  // Additional initialization (change thresholds on keys, ...) 
  ExtraCode_Init();

  // Executive loop, runs continually after initialization 
  for (;;)
  {
    
    // Menu update managemnt 
    Key = ReadJoystick();
    
    if ((Key != J_NONE) && (Key != J_RELEASED))
    {
      if (IntroSkipped)
      {
        IntroSkipped = 0;
      }
      else
      {
        MenuUpdate(Key);
      }
    }
   
    // Execute Touch Sensing layer 
    TSL_Action();
   
    // Execute User Application 
    Display_Action();
    KeyPad_Action();
    SystemRecovery_Action();
		LowPower_Action();
    
  }
}

/**
  ******************************************************************************
  * @Sets the keys configuration according to the user application.
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
void ExtraCode_Init(void)
{

  u8 i;

  // All keys and slider are implemented and enabled 

#if NUMBER_OF_SINGLE_CHANNEL_KEYS > 0
  for (i = 0; i < NUMBER_OF_SINGLE_CHANNEL_KEYS; i++)
  {
    sSCKeyInfo[i].Setting.b.IMPLEMENTED = 1;
    sSCKeyInfo[i].Setting.b.ENABLED = 1;
    sSCKeyInfo[i].DxSGroup = 0x00;
  }
#endif

#if NUMBER_OF_MULTI_CHANNEL_KEYS > 0
  for (i = 0; i < NUMBER_OF_MULTI_CHANNEL_KEYS; i++)
  {
    sMCKeyInfo[i].Setting.b.IMPLEMENTED = 1;
    sMCKeyInfo[i].Setting.b.ENABLED = 1;
    sMCKeyInfo[i].DxSGroup = 0x00;
  }
#endif

  // Adjust key per key the sensitivity thresholds according to user feeling  

#if defined(BOARD_STMT_8L_EV1)
  // K0 
	KEY0.DetectThreshold += 0;
	KEY0.EndDetectThreshold += 0;
	KEY0.RecalibrationThreshold += 0;
	// K1 
	KEY1.DetectThreshold += 0;
	KEY1.EndDetectThreshold += 0;
	KEY1.RecalibrationThreshold += 0;
	// K2 
	KEY2.DetectThreshold += 8;
	KEY2.EndDetectThreshold += 8;
	KEY2.RecalibrationThreshold += 8;
	// K3 
	KEY3.DetectThreshold += 0;
	KEY3.EndDetectThreshold += 0;
	KEY3.RecalibrationThreshold += 0;
	// K4 
	KEY4.DetectThreshold += 5;
	KEY4.EndDetectThreshold += 5;
	KEY4.RecalibrationThreshold += 5;
	// K5 
	KEY5.DetectThreshold += 8;
	KEY5.EndDetectThreshold += 8;
	KEY5.RecalibrationThreshold += 8;
	// K6 
	KEY6.DetectThreshold += 0;
	KEY6.EndDetectThreshold += 0;
	KEY6.RecalibrationThreshold += 0;
	// K7 
	KEY7.DetectThreshold += 0;
	KEY7.EndDetectThreshold += 0;
	KEY7.RecalibrationThreshold += 0;
	// K8 
	KEY8.DetectThreshold += 5;
	KEY8.EndDetectThreshold += 5;
	KEY8.RecalibrationThreshold += 5;
	// K9 
	KEY9.DetectThreshold += 0;
	KEY9.EndDetectThreshold += 0;
	KEY9.RecalibrationThreshold += 0;
  // Wheel 
	WHEEL.DetectThreshold += 0;
	WHEEL.EndDetectThreshold += 0;
	WHEEL.RecalibrationThreshold += 0;  
  // Slider 
	SLIDER.DetectThreshold += 10;
	SLIDER.EndDetectThreshold += 10;
	SLIDER.RecalibrationThreshold += 10;
#endif

  enableInterrupts();

}

/**
  ******************************************************************************
  * @brief Reports the name of the source file and the source line number where
  * the assert error has occurred.
  * User can add his own implementation to report the file name and line number.
  * ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line)
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  ******************************************************************************
  */
#ifdef FULL_ASSERT
void assert_failed(u8 *file, u16 line)
#else
void assert_failed(void)
#endif
{
  // Put enter your own code to manage an assert error 
  for (;;)
  {
  }
}

 // (C) COPYRIGHT 2010 STMicroelectronics *****END OF FILE***
