/**
  ******************************************************************************
  * @file    menu.c
  * @brief   This file contains the main function.
  * @author  STMicroelectronics - MCD Application Team
  * @version V1.2.0
  * @date    04-NOV-2010
  ******************************************************************************
  */

// Includes ------------------------------------------------------------------
#include "menu.h"

// Private Constant ----------------------------------------------------------

#if defined(BOARD_STMT_8L_EV1)
__CONST u8 *DemoGeneric = "*** STMT/8L-EV1 EVALUATION BOARD ***";
#elif defined(BOARD_STM8L151CxT_TS1)
__CONST u8 *DemoGeneric = "*** STM8L151CxT-TS1 EVALUATION BOARD ***";
#endif

__CONST MENU_NEXTSTATE_T MENU_NEXTSTATE[] =
  {
    
    // State               Key             NextState
    
    //==========================================================================
    // MENU Level 1
    //==========================================================================
    
    { HELP,                LEFT,           ABOUT },
    { HELP,                RIGHT,          KEY_INFO },
    { HELP,                DOWN,           EXE_HELP }, // To MENU Execution
   
    { KEY_INFO,            LEFT,           HELP },
    { KEY_INFO,            RIGHT,          DETECTION_TIMEOUT },
    { KEY_INFO,            DOWN,           EXE_KEYS_SUMMARY_DISPLAY }, // To MENU Level 2
   
    { DETECTION_TIMEOUT,   LEFT,           KEY_INFO },
    { DETECTION_TIMEOUT,   RIGHT,          DEBOUNCE_FILTER },
    { DETECTION_TIMEOUT,   DOWN,           EXE_DETECTION_TIMEOUT_DISPLAY }, // To MENU Execution
   
    { DEBOUNCE_FILTER,     LEFT,           DETECTION_TIMEOUT },
    { DEBOUNCE_FILTER,     RIGHT,          POWER_SAVING_MODE },
    { DEBOUNCE_FILTER,     DOWN,           EXE_DEBOUNCE_FILTER_DISPLAY }, // To MENU Execution
   
    { POWER_SAVING_MODE,   LEFT,           DEBOUNCE_FILTER },
    { POWER_SAVING_MODE,   RIGHT,          DxS_SETTING },
    { POWER_SAVING_MODE,   DOWN,           EXE_POWER_SAVING_MODE_DISPLAY }, // To MENU Execution
   
    { DxS_SETTING,         LEFT,           POWER_SAVING_MODE },
    { DxS_SETTING,         RIGHT,          ABOUT },
    { DxS_SETTING,         DOWN,           EXE_DxS_SETTING_DISPLAY }, // To MENU Execution
   
    { ABOUT,               LEFT,           DxS_SETTING },
    { ABOUT,               RIGHT,          HELP },
    { ABOUT,               DOWN,           EXE_ABOUT }, // To MENU Execution
   
    //==========================================================================
    // MENU Level 2
    //==========================================================================
   
    { EXE_KEYS_SUMMARY_DISPLAY,      UP,             KEY_INFO },
    { EXE_KEYS_SUMMARY_DISPLAY,      LEFT,           Wheel_SELECTION },
    { EXE_KEYS_SUMMARY_DISPLAY,      RIGHT,          KEY0KEY9_SELECTION },
    
    { KEY0KEY9_SELECTION,            UP,             KEY_INFO },
    { KEY0KEY9_SELECTION,            LEFT,           EXE_KEYS_SUMMARY_DISPLAY },
    { KEY0KEY9_SELECTION,            RIGHT,          Slider_SELECTION },
    { KEY0KEY9_SELECTION,            DOWN,           KEY9_SIGNAL_REFERENCE }, // To MENU Level 3a
    
    { Slider_SELECTION,              UP,             KEY_INFO },
    { Slider_SELECTION,              LEFT,           KEY0KEY9_SELECTION },
    { Slider_SELECTION,              RIGHT,          Wheel_SELECTION },
    { Slider_SELECTION,              DOWN,           Slider_STATE }, // To MENU Level 3b
    
    { Wheel_SELECTION,               UP,             KEY_INFO },   
    { Wheel_SELECTION,               LEFT,           Slider_SELECTION },
    { Wheel_SELECTION,               RIGHT,          EXE_KEYS_SUMMARY_DISPLAY },
    { Wheel_SELECTION,               DOWN,           Wheel_STATE }, // To MENU Level 3c
    
    //==========================================================================
    // MENU Level 3a (from KEY0KEY9_SELECTION)
    //==========================================================================

    { KEY0_SIGNAL_REFERENCE,     UP,             KEY0KEY9_SELECTION },
    { KEY0_SIGNAL_REFERENCE,     LEFT,           KEY9_SIGNAL_REFERENCE },
    { KEY0_SIGNAL_REFERENCE,     RIGHT,          KEY1_SIGNAL_REFERENCE },

    { KEY1_SIGNAL_REFERENCE,     UP,             KEY0KEY9_SELECTION },
    { KEY1_SIGNAL_REFERENCE,     LEFT,           KEY0_SIGNAL_REFERENCE },
    { KEY1_SIGNAL_REFERENCE,     RIGHT,          KEY2_SIGNAL_REFERENCE },

    { KEY2_SIGNAL_REFERENCE,     UP,             KEY0KEY9_SELECTION },
    { KEY2_SIGNAL_REFERENCE,     LEFT,           KEY1_SIGNAL_REFERENCE },
    { KEY2_SIGNAL_REFERENCE,     RIGHT,          KEY3_SIGNAL_REFERENCE },

    { KEY3_SIGNAL_REFERENCE,     UP,             KEY0KEY9_SELECTION },
    { KEY3_SIGNAL_REFERENCE,     LEFT,           KEY2_SIGNAL_REFERENCE },
    { KEY3_SIGNAL_REFERENCE,     RIGHT,          KEY4_SIGNAL_REFERENCE },

    { KEY4_SIGNAL_REFERENCE,     UP,             KEY0KEY9_SELECTION },
    { KEY4_SIGNAL_REFERENCE,     LEFT,           KEY3_SIGNAL_REFERENCE },
    { KEY4_SIGNAL_REFERENCE,     RIGHT,          KEY5_SIGNAL_REFERENCE },

    { KEY5_SIGNAL_REFERENCE,     UP,             KEY0KEY9_SELECTION },
    { KEY5_SIGNAL_REFERENCE,     LEFT,           KEY4_SIGNAL_REFERENCE },
    { KEY5_SIGNAL_REFERENCE,     RIGHT,          KEY6_SIGNAL_REFERENCE },

    { KEY6_SIGNAL_REFERENCE,     UP,             KEY0KEY9_SELECTION },
    { KEY6_SIGNAL_REFERENCE,     LEFT,           KEY5_SIGNAL_REFERENCE },
    { KEY6_SIGNAL_REFERENCE,     RIGHT,          KEY7_SIGNAL_REFERENCE },

    { KEY7_SIGNAL_REFERENCE,     UP,             KEY0KEY9_SELECTION },
    { KEY7_SIGNAL_REFERENCE,     LEFT,           KEY6_SIGNAL_REFERENCE },
    { KEY7_SIGNAL_REFERENCE,     RIGHT,          KEY8_SIGNAL_REFERENCE },

    { KEY8_SIGNAL_REFERENCE,     UP,             KEY0KEY9_SELECTION },
    { KEY8_SIGNAL_REFERENCE,     LEFT,           KEY7_SIGNAL_REFERENCE },
    { KEY8_SIGNAL_REFERENCE,     RIGHT,          KEY9_SIGNAL_REFERENCE },

    { KEY9_SIGNAL_REFERENCE,     UP,             KEY0KEY9_SELECTION },
    { KEY9_SIGNAL_REFERENCE,     LEFT,           KEY8_SIGNAL_REFERENCE },
    { KEY9_SIGNAL_REFERENCE,     RIGHT,          KEY0_SIGNAL_REFERENCE },
    { KEY9_SIGNAL_REFERENCE,     DOWN,           KEY9_DETECT_TH }, // To MENU Level 4a

    //==========================================================================
    // MENU Level 3b (from Slider_SELECTION)
    //==========================================================================

    { Slider_STATE,                UP,        Slider_SELECTION },
    { Slider_STATE,                LEFT,      Slider_RESOLUTION_DIRECTION },
    { Slider_STATE,                RIGHT,     Slider_SIGNAL_REFERENCE },
    
    { Slider_SIGNAL_REFERENCE,     UP,        Slider_SELECTION },
    { Slider_SIGNAL_REFERENCE,     LEFT,      Slider_STATE },
    { Slider_SIGNAL_REFERENCE,     RIGHT,     Slider_SIGNAL_POSITION },
    
    { Slider_SIGNAL_POSITION,      UP,        Slider_SELECTION },
    { Slider_SIGNAL_POSITION,      LEFT,      Slider_SIGNAL_REFERENCE },
    { Slider_SIGNAL_POSITION,      RIGHT,     Slider_RESOLUTION_DIRECTION },
    
    { Slider_RESOLUTION_DIRECTION, UP,        Slider_SELECTION },
    { Slider_RESOLUTION_DIRECTION, LEFT,      Slider_SIGNAL_POSITION },
    { Slider_RESOLUTION_DIRECTION, RIGHT,     Slider_STATE },
    { Slider_RESOLUTION_DIRECTION, DOWN,      Slider_RESOLUTION_CHANGE }, // To MENU Level 4b
    
    //==========================================================================
    // MENU Level 3c (from Wheel_SELECTION)
    //==========================================================================

    { Wheel_STATE,                 UP,        Wheel_SELECTION },
    { Wheel_STATE,                 LEFT,      Wheel_RESOLUTION_DIRECTION },
    { Wheel_STATE,                 RIGHT,     Wheel_SIGNAL_REFERENCE },

    { Wheel_SIGNAL_REFERENCE,      UP,        Wheel_SELECTION },
    { Wheel_SIGNAL_REFERENCE,      LEFT,      Wheel_STATE },
    { Wheel_SIGNAL_REFERENCE,      RIGHT,     Wheel_SIGNAL_POSITION },    
    
    { Wheel_SIGNAL_POSITION,       UP,        Wheel_SELECTION },
    { Wheel_SIGNAL_POSITION,       LEFT,      Wheel_SIGNAL_REFERENCE },
    { Wheel_SIGNAL_POSITION,       RIGHT,     Wheel_RESOLUTION_DIRECTION },
    
    { Wheel_RESOLUTION_DIRECTION,  UP,        Wheel_SELECTION },
    { Wheel_RESOLUTION_DIRECTION,  LEFT,      Wheel_SIGNAL_POSITION },
    { Wheel_RESOLUTION_DIRECTION,  RIGHT,     Wheel_STATE },
    { Wheel_RESOLUTION_DIRECTION,  DOWN,      Wheel_RESOLUTION_CHANGE }, // To MENU Level 4c

    //==========================================================================
    // MENU Level 4a (from KEY9_SIGNAL_REFERENCE)
    //==========================================================================
   
    { KEY9_DETECT_TH,            UP,             KEY9_SIGNAL_REFERENCE },
    { KEY9_DETECT_TH,            LEFT,           KEY9_RECALIB_TH },
    { KEY9_DETECT_TH,            RIGHT,          KEY9_ENDDETECT_TH },
    { KEY9_DETECT_TH,            DOWN,           KEY9_DETECT_TH_CHANGE }, // To MENU Level 5a

    { KEY9_ENDDETECT_TH,         UP,             KEY9_SIGNAL_REFERENCE },
    { KEY9_ENDDETECT_TH,         LEFT,           KEY9_DETECT_TH },
    { KEY9_ENDDETECT_TH,         RIGHT,          KEY9_RECALIB_TH },
    { KEY9_ENDDETECT_TH,         DOWN,           KEY9_ENDDETECT_TH_CHANGE }, // To MENU Level 5b

    { KEY9_RECALIB_TH,           UP,             KEY9_SIGNAL_REFERENCE },
    { KEY9_RECALIB_TH,           LEFT,           KEY9_ENDDETECT_TH },
    { KEY9_RECALIB_TH,           RIGHT,          KEY9_DETECT_TH },
    { KEY9_RECALIB_TH,           DOWN,           KEY9_RECALIB_TH_CHANGE }, // To MENU Level 5c

    //==========================================================================
    // MENU Level 4b (from Slider_RESOLUTION_DIRECTION)
    //==========================================================================

    { Slider_RESOLUTION_CHANGE,    UP,        Slider_RESOLUTION_DIRECTION },
    { Slider_RESOLUTION_CHANGE,    LEFT,      EXE_Slider_DEC_SW_RESOLUTION },
    { Slider_RESOLUTION_CHANGE,    RIGHT,     EXE_Slider_INC_SW_RESOLUTION },

    //==========================================================================
    // MENU Level 4c (from Wheel_RESOLUTION_DIRECTION)
    //==========================================================================

    { Wheel_RESOLUTION_CHANGE,     UP,        Wheel_RESOLUTION_DIRECTION },
    { Wheel_RESOLUTION_CHANGE,     LEFT,      EXE_Wheel_DEC_SW_RESOLUTION },
    { Wheel_RESOLUTION_CHANGE,     RIGHT,     EXE_Wheel_INC_SW_RESOLUTION },

    //==========================================================================
    // MENU Level 5a (from KEY9_DETECT_TH)
    //==========================================================================
    
    { KEY9_DETECT_TH_CHANGE,       UP,        KEY9_DETECT_TH },
    { KEY9_DETECT_TH_CHANGE,       LEFT,      EXE_KEY9_DEC_DETECT_TH },
    { KEY9_DETECT_TH_CHANGE,       RIGHT,     EXE_KEY9_INC_DETECT_TH },

    //==========================================================================
    // MENU Level 5b (from KEY9_ENDDETECT_TH)
    //==========================================================================

    { KEY9_ENDDETECT_TH_CHANGE,    UP,        KEY9_ENDDETECT_TH },
    { KEY9_ENDDETECT_TH_CHANGE,    LEFT,      EXE_KEY9_DEC_ENDDETECT_TH },
    { KEY9_ENDDETECT_TH_CHANGE,    RIGHT,     EXE_KEY9_INC_ENDDETECT_TH },
    
    //==========================================================================
    // MENU Level 5c (from KEY9_RECALIB_TH)
    //==========================================================================

    { KEY9_RECALIB_TH_CHANGE,      UP,        KEY9_RECALIB_TH },
    { KEY9_RECALIB_TH_CHANGE,      LEFT,      EXE_KEY9_DEC_RECALIB_TH },
    { KEY9_RECALIB_TH_CHANGE,      RIGHT,     EXE_KEY9_INC_RECALIB_TH },
    
    //==========================================================================
    // MENU Execution
    //==========================================================================
    
    { EXE_HELP,                        UP,       HELP },
    
    { EXE_DETECTION_TIMEOUT_DISPLAY,   UP,       DETECTION_TIMEOUT },
    { EXE_DETECTION_TIMEOUT_DISPLAY,   LEFT,     EXE_DEC_DETECTION_TIMEOUT },
    { EXE_DETECTION_TIMEOUT_DISPLAY,   RIGHT,    EXE_INC_DETECTION_TIMEOUT },
    { EXE_DEC_DETECTION_TIMEOUT,       UP,       DETECTION_TIMEOUT },
    { EXE_DEC_DETECTION_TIMEOUT,       LEFT,     EXE_DEC_DETECTION_TIMEOUT },
    { EXE_DEC_DETECTION_TIMEOUT,       RIGHT,    EXE_INC_DETECTION_TIMEOUT },
    { EXE_INC_DETECTION_TIMEOUT,       UP,       DETECTION_TIMEOUT },
    { EXE_INC_DETECTION_TIMEOUT,       LEFT,     EXE_DEC_DETECTION_TIMEOUT },
    { EXE_INC_DETECTION_TIMEOUT,       RIGHT,    EXE_INC_DETECTION_TIMEOUT },

    { EXE_DEBOUNCE_FILTER_DISPLAY,     UP,       DEBOUNCE_FILTER },
    { EXE_DEBOUNCE_FILTER_DISPLAY,     LEFT,     EXE_DEC_DEBOUNCE_FILTER },
    { EXE_DEBOUNCE_FILTER_DISPLAY,     RIGHT,    EXE_INC_DEBOUNCE_FILTER },
    { EXE_DEC_DEBOUNCE_FILTER,         UP,       DEBOUNCE_FILTER },
    { EXE_DEC_DEBOUNCE_FILTER,         LEFT,     EXE_DEC_DEBOUNCE_FILTER },
    { EXE_DEC_DEBOUNCE_FILTER,         RIGHT,    EXE_INC_DEBOUNCE_FILTER },
    { EXE_INC_DEBOUNCE_FILTER,         UP,       DEBOUNCE_FILTER },
    { EXE_INC_DEBOUNCE_FILTER,         LEFT,     EXE_DEC_DEBOUNCE_FILTER },
    { EXE_INC_DEBOUNCE_FILTER,         RIGHT,    EXE_INC_DEBOUNCE_FILTER },

    { EXE_POWER_SAVING_MODE_DISPLAY,   UP,       POWER_SAVING_MODE },
    { EXE_POWER_SAVING_MODE_DISPLAY,   LEFT,     EXE_DEC_POWER_SAVING_MODE },
    { EXE_POWER_SAVING_MODE_DISPLAY,   RIGHT,    EXE_INC_POWER_SAVING_MODE },
    { EXE_INC_POWER_SAVING_MODE,       UP,       POWER_SAVING_MODE },
    { EXE_INC_POWER_SAVING_MODE,       LEFT,     EXE_DEC_POWER_SAVING_MODE },
    { EXE_INC_POWER_SAVING_MODE,       RIGHT,    EXE_INC_POWER_SAVING_MODE },
    { EXE_DEC_POWER_SAVING_MODE,       UP,       POWER_SAVING_MODE },
    { EXE_DEC_POWER_SAVING_MODE,       LEFT,     EXE_DEC_POWER_SAVING_MODE },
    { EXE_DEC_POWER_SAVING_MODE,       RIGHT,    EXE_INC_POWER_SAVING_MODE },

    { EXE_DxS_SETTING_DISPLAY,         UP,       DxS_SETTING },
    { EXE_DxS_SETTING_DISPLAY,         LEFT,     EXE_DEC_DxS },
    { EXE_DxS_SETTING_DISPLAY,         RIGHT,    EXE_INC_DxS },
    { EXE_INC_DxS,                     UP,       DxS_SETTING },
    { EXE_INC_DxS,                     LEFT,     EXE_DEC_DxS },
    { EXE_INC_DxS,                     RIGHT,    EXE_INC_DxS },
    { EXE_DEC_DxS,                     UP,       DxS_SETTING },
    { EXE_DEC_DxS,                     LEFT,     EXE_DEC_DxS },
    { EXE_DEC_DxS,                     RIGHT,    EXE_INC_DxS },

    { EXE_ABOUT,                       UP,       ABOUT },

    { EXE_KEY9_DEC_DETECT_TH,          UP,       KEY9_DETECT_TH },
    { EXE_KEY9_DEC_DETECT_TH,          LEFT,     EXE_KEY9_DEC_DETECT_TH },
    { EXE_KEY9_DEC_DETECT_TH,          RIGHT,    EXE_KEY9_INC_DETECT_TH },
    { EXE_KEY9_INC_DETECT_TH,          UP,       KEY9_DETECT_TH },
    { EXE_KEY9_INC_DETECT_TH,          LEFT,     EXE_KEY9_DEC_DETECT_TH },
    { EXE_KEY9_INC_DETECT_TH,          RIGHT,    EXE_KEY9_INC_DETECT_TH },
   
    { EXE_KEY9_DEC_ENDDETECT_TH,       UP,       KEY9_ENDDETECT_TH },
    { EXE_KEY9_DEC_ENDDETECT_TH,       LEFT,     EXE_KEY9_DEC_ENDDETECT_TH },
    { EXE_KEY9_DEC_ENDDETECT_TH,       RIGHT,    EXE_KEY9_INC_ENDDETECT_TH },
    { EXE_KEY9_INC_ENDDETECT_TH,       UP,       KEY9_ENDDETECT_TH },
    { EXE_KEY9_INC_ENDDETECT_TH,       LEFT,     EXE_KEY9_DEC_ENDDETECT_TH },
    { EXE_KEY9_INC_ENDDETECT_TH,       RIGHT,    EXE_KEY9_INC_ENDDETECT_TH },
    
    { EXE_KEY9_DEC_RECALIB_TH,         UP,       KEY9_RECALIB_TH },
    { EXE_KEY9_DEC_RECALIB_TH,         LEFT,     EXE_KEY9_DEC_RECALIB_TH },
    { EXE_KEY9_DEC_RECALIB_TH,         RIGHT,    EXE_KEY9_INC_RECALIB_TH },
    { EXE_KEY9_INC_RECALIB_TH,         UP,       KEY9_RECALIB_TH },
    { EXE_KEY9_INC_RECALIB_TH,         LEFT,     EXE_KEY9_DEC_RECALIB_TH },
    { EXE_KEY9_INC_RECALIB_TH,         RIGHT,    EXE_KEY9_INC_RECALIB_TH },
    
    { EXE_Slider_DEC_SW_RESOLUTION,    UP,       Slider_RESOLUTION_DIRECTION },
    { EXE_Slider_DEC_SW_RESOLUTION,    LEFT,     EXE_Slider_DEC_SW_RESOLUTION },
    { EXE_Slider_DEC_SW_RESOLUTION,    RIGHT,    EXE_Slider_INC_SW_RESOLUTION },
    { EXE_Slider_INC_SW_RESOLUTION,    UP,       Slider_RESOLUTION_DIRECTION},
    { EXE_Slider_INC_SW_RESOLUTION,    LEFT,     EXE_Slider_DEC_SW_RESOLUTION },
    { EXE_Slider_INC_SW_RESOLUTION,    RIGHT,    EXE_Slider_INC_SW_RESOLUTION },
    
    { EXE_Wheel_DEC_SW_RESOLUTION,     UP,       Wheel_RESOLUTION_DIRECTION },
    { EXE_Wheel_DEC_SW_RESOLUTION,     LEFT,     EXE_Wheel_DEC_SW_RESOLUTION },
    { EXE_Wheel_DEC_SW_RESOLUTION,     RIGHT,    EXE_Wheel_INC_SW_RESOLUTION },
    { EXE_Wheel_INC_SW_RESOLUTION,     UP,       Wheel_RESOLUTION_DIRECTION },
    { EXE_Wheel_INC_SW_RESOLUTION,     LEFT,     EXE_Wheel_DEC_SW_RESOLUTION },
    { EXE_Wheel_INC_SW_RESOLUTION,     RIGHT,    EXE_Wheel_INC_SW_RESOLUTION },

    // End of table
    { INIT,                            J_NONE,   INIT }
  };

__CONST MENU_STATE_T MENU_STATE[] =
  {
    
    // STATE                          STATE TEXT                   FUNCTION 

    //==========================================================================
    // MENUs with TEXT
    //==========================================================================
   
    { HELP,                           HELP_TEXT,                   0 },
    { KEY_INFO,                       KEY_INFO_TEXT,               0 },
    { DETECTION_TIMEOUT,              DETECTION_TIMEOUT_TEXT,      0 },
    { DEBOUNCE_FILTER,                DEBOUNCE_FILTER_TEXT,        0 },
    { POWER_SAVING_MODE,              POWER_SAVING_MODE_TEXT,      0 },
    { DxS_SETTING,                    DxS_SETTING_TEXT,            0 },
    { ABOUT,                          ABOUT_TEXT,                  0 },

    { KEY0KEY9_SELECTION,             KEY0KEY9_SELECTION_TEXT,     0 },
    { Slider_SELECTION,               Slider_SELECTION_TEXT,       0 },
    { Wheel_SELECTION,                Wheel_SELECTION_TEXT,        0 },

    //==========================================================================
    // MENUs with Execution function
    //==========================================================================

    { EXE_HELP,                       0,  HELP_Exec },
    
    { EXE_DETECTION_TIMEOUT_DISPLAY,  0,  DETECTION_TIMEOUT_Exec },
    { EXE_DEC_DETECTION_TIMEOUT,      0,  DEC_DETECTION_TIMEOUT_Exec },
    { EXE_INC_DETECTION_TIMEOUT,      0,  INC_DETECTION_TIMEOUT_Exec },
    
    { EXE_DEBOUNCE_FILTER_DISPLAY,    0,  DEBOUNCE_FILTER_Exec },
    { EXE_DEC_DEBOUNCE_FILTER,        0,  DEC_DEBOUNCE_FILTER_Exec },
    { EXE_INC_DEBOUNCE_FILTER,        0,  INC_DEBOUNCE_FILTER_Exec },
    
    { EXE_POWER_SAVING_MODE_DISPLAY,  0,  POWER_SAVING_MODE_Exec},
    { EXE_DEC_POWER_SAVING_MODE,      0,  DEC_POWER_SAVING_Exec },
    { EXE_INC_POWER_SAVING_MODE,      0,  INC_POWER_SAVING_Exec },
    
    { EXE_DxS_SETTING_DISPLAY,        0,  DxS_SETTING_Exec },
    { EXE_DEC_DxS,                    0,  DxS_DEC_Exec },
    { EXE_INC_DxS,                    0,  DxS_INC_Exec },
   
    { EXE_ABOUT,                      0,  ABOUT_Exec },
   
    { EXE_KEYS_SUMMARY_DISPLAY,       0,  KEYS_SUMMARY_DISPLAY_Exec },
   
    { KEY0_SIGNAL_REFERENCE,          0,  KEY0_SIGNAL_REFERENCE_Exec },
    { KEY1_SIGNAL_REFERENCE,          0,  KEY1_SIGNAL_REFERENCE_Exec },
    { KEY2_SIGNAL_REFERENCE,          0,  KEY2_SIGNAL_REFERENCE_Exec },
    { KEY3_SIGNAL_REFERENCE,          0,  KEY3_SIGNAL_REFERENCE_Exec },
    { KEY4_SIGNAL_REFERENCE,          0,  KEY4_SIGNAL_REFERENCE_Exec },
    { KEY5_SIGNAL_REFERENCE,          0,  KEY5_SIGNAL_REFERENCE_Exec },
    { KEY6_SIGNAL_REFERENCE,          0,  KEY6_SIGNAL_REFERENCE_Exec },
    { KEY7_SIGNAL_REFERENCE,          0,  KEY7_SIGNAL_REFERENCE_Exec },
    { KEY8_SIGNAL_REFERENCE,          0,  KEY8_SIGNAL_REFERENCE_Exec },
    { KEY9_SIGNAL_REFERENCE,          0,  KEY9_SIGNAL_REFERENCE_Exec },
    
    { KEY9_DETECT_TH,                 0,  KEY9_DETECT_TH_Exec },
    { KEY9_DETECT_TH_CHANGE,          0,  KEY9_DETECT_TH_CHANGE_Exec },
    { EXE_KEY9_INC_DETECT_TH,         0,  KEY9_DETECT_TH_INC_Exec },
    { EXE_KEY9_DEC_DETECT_TH,         0,  KEY9_DETECT_TH_DEC_Exec },
   
    { KEY9_ENDDETECT_TH,              0,  KEY9_ENDDETECT_TH_Exec },
    { KEY9_ENDDETECT_TH_CHANGE,       0,  KEY9_ENDDETECT_TH_CHANGE_Exec },
    { EXE_KEY9_INC_ENDDETECT_TH,      0,  KEY9_ENDDETECT_TH_INC_Exec },
    { EXE_KEY9_DEC_ENDDETECT_TH,      0,  KEY9_ENDDETECT_TH_DEC_Exec },
    
    { KEY9_RECALIB_TH,                0,  KEY9_RECALIB_TH_Exec },
    { KEY9_RECALIB_TH_CHANGE,         0,  KEY9_RECALIB_TH_CHANGE_Exec },
    { EXE_KEY9_INC_RECALIB_TH,        0,  KEY9_RECALIB_TH_INC_Exec },
    { EXE_KEY9_DEC_RECALIB_TH,        0,  KEY9_RECALIB_TH_DEC_Exec },
    
    { Slider_STATE,                   0,  Slider_STATE_Exec },
    { Slider_SIGNAL_REFERENCE,        0,  Slider_SIGNAL_REFERENCE_Exec },
    { Slider_SIGNAL_POSITION,         0,  Slider_SIGNAL_POSITION_Exec },
    { Slider_RESOLUTION_DIRECTION,    0,  Slider_RESOLUTION_DIRECTION_Exec },
    { Slider_RESOLUTION_CHANGE,       0,  Slider_RESOLUTION_CHANGE_Exec },
    { EXE_Slider_DEC_SW_RESOLUTION,   0,  Slider_DEC_SW_RESOLUTION_Exec},
    { EXE_Slider_INC_SW_RESOLUTION,   0,  Slider_INC_SW_RESOLUTION_Exec },
   
    { Wheel_STATE,                    0,  Wheel_STATE_Exec },
    { Wheel_SIGNAL_REFERENCE,         0,  Wheel_SIGNAL_REFERENCE_Exec },
    { Wheel_SIGNAL_POSITION,          0,  Wheel_SIGNAL_POSITION_Exec },
    { Wheel_RESOLUTION_DIRECTION,     0,  Wheel_RESOLUTION_DIRECTION_Exec },
    { Wheel_RESOLUTION_CHANGE,        0,  Wheel_RESOLUTION_CHANGE_Exec },
    { EXE_Wheel_DEC_SW_RESOLUTION,    0,  Wheel_DEC_SW_RESOLUTION_Exec},
    { EXE_Wheel_INC_SW_RESOLUTION,    0,  Wheel_INC_SW_RESOLUTION_Exec },

    // End of table
    { INIT,                           0,  0 }
    
  };

// Private typedef -----------------------------------------------------------
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
u8 HelpMessageIndex = 0;
STATE_ID_T PreviousTopState, State;
void (*pStateFunc)(void);

// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

/**
  ******************************************************************************
  * Initialize the menu variables. Must be called by application Init function.
  * @param[in] None
  * @return None
  ******************************************************************************
  */
void MenuInit(void)
{
  PreviousTopState = EXE_KEYS_SUMMARY_DISPLAY;
  State = EXE_KEYS_SUMMARY_DISPLAY;
  pStateFunc = 0;
}

/**
  ******************************************************************************
  * This function updates the menu states and display the new menu on the LCD.
  * @param[in] ujoystick is the new joystick state
  * @return None
  ******************************************************************************
  */
void MenuUpdate(JOYSTICK_ID_T ujoystick)
{

  u16 i;
  STATE_ID_T NextState;
 
  NextState = INIT;

  // Search if the next menu state exists 
  for (i = 0; MENU_NEXTSTATE[i].State; i++)
  {
    if ((MENU_NEXTSTATE[i].State == State) && (MENU_NEXTSTATE[i].Key == ujoystick))
    {
      NextState = MENU_NEXTSTATE[i].NextState;
      break;
    }
  }

  // if next state exists 
  if (NextState)
  {
    // Update the previous top state 
    switch (ujoystick)
    {
      case UP:
        PreviousTopState = INIT;
        for (i = 0; MENU_NEXTSTATE[i].State; i++)
        {
          if ((MENU_NEXTSTATE[i].State == NextState) && (MENU_NEXTSTATE[i].Key == UP))
          {
            PreviousTopState = MENU_NEXTSTATE[i].NextState;
            break;
          }
        }
        break;

      case SEL:
        // If a sub-menu is entered, the PreviousTopState is the current state 
        PreviousTopState = State;
        break;

      default:
        break;
    }
    
    State = NextState; /* Update current state */

    MenuDisplay(); /* Display the new menu on LCD */

    // Execute the function of state if there is one
    for (i = 0; MENU_STATE[i].State; i++)
    {
      if (MENU_STATE[i].State == State)
      {
        pStateFunc = MENU_STATE[i].pFunc;
        if (pStateFunc) /* Check if there is a state function to be executed */
        {
          pStateFunc(); /* Execute the state function */
          pStateFunc = 0;
        }
      }
    }
  }
}

/**
  ******************************************************************************
  * This function displays the current menu level on the LCD
  * @param[in] None
  * @return None
  ******************************************************************************
  */
void MenuDisplay(void)
{

  u8 i;
  
  // Display menu line 1 
  if (PreviousTopState)
  {
    for (i = 0; MENU_STATE[i].State; i++)
    {
      if (MENU_STATE[i].State == PreviousTopState)
      {
        if (MENU_STATE[i].pText)
        {
          //LCD_Clear();
          LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, MENU_STATE[i].pText);
         
        }
        break;
      }
    }
  }
  else
  {
    // Menu top level reached 
    LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, MENU_TOP_TEXT);
  }

  // Display menu line 2 
  for (i = 0; MENU_STATE[i].State; i++)
  {
    if (MENU_STATE[i].State == State)
    {
      if (MENU_STATE[i].pText)
      {
        LCD_Clear();
        LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, MENU_TOP_TEXT);
        LCD_PrintString(LCD_LINE2, ENABLE, DISABLE, MENU_STATE[i].pText);
      }
      break;
    }
  }
  
}

/**
  ******************************************************************************
  * This function displays the wellcome rolling text message
  * @param[in] None
  * @return u8 1=display skipped
  ******************************************************************************
  */
u8 MenuDisplayWelcomeMessage(void)
{
  // Display the welcome text on first LCD line 
  LCD_ClearLine(LCD_LINE2);
  LCD_DisplayLogo(0x93);
  return(LCD_RollString(LCD_LINE1, DemoGeneric, 0xFFFF));
}

/**
  ******************************************************************************
  * This function displays the "about" menu string
  * @param[in] None
  * @return None
  ******************************************************************************
  */
void ABOUT_Exec(void)
{
  
  LCD_Clear();

#if defined(BOARD_STMT_8L_EV1)
  LCD_PrintString(LCD_LINE1, ENABLE, DISABLE,  "  STMT/8L-EV1  ");
  LCD_PrintString(LCD_LINE2, DISABLE, DISABLE, "     v1.2.0");
#elif defined(BOARD_STM8L151CxT_TS1)
  LCD_PrintString(LCD_LINE1, ENABLE, DISABLE,  "STM8L151CxT-TS1");
  LCD_PrintString(LCD_LINE2, DISABLE, DISABLE, "EVAL v2.3.0");
#endif

#if defined(_COSMIC_) || defined(__CSMC__)
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "   C");
#elif defined(_RAISONANCE_) || defined(__RCST7__)
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "   R");
#elif defined(__ICCSTM8__) || defined(_IAR_)
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "   I");
#else
  LCD_PrintString(LCD_LINE2, DISABLE, ENABLE, "   ?");
#endif

}

/**
  ******************************************************************************
  * Update the help message if EXEHELP state selected
  * @param[in] None
  * @return None
  ******************************************************************************
  */
void HELP_Exec(void)
{

  JOYSTICK_ID_T Key;

  HelpMessageIndex = 0;
  LCD_PrintMsg(HELP_LINE1_TEXT);

  GetTimeOutStartTime();

  for (;;)
  {
    //IWDG_ReloadCounter(); /* reload counter value to max */
    Key = ReadJoystick();
    TSL_Action();
    if ((Key == UP) || (sSCKeyInfo[2].Setting.b.DETECTED))
    {
      MenuUpdate(Key);
      return;
    }

    if (TimeOut(HELP_PERIOD) == FALSE)
    {
      // do nothing 
    }
    else
    {
      GetTimeOutStartTime();
      TSL_Action();

      HelpMessageIndex++;
      if (HelpMessageIndex > 2)
      {
        HelpMessageIndex = 0;
      }
      switch (HelpMessageIndex)
      {
        case 0:
          LCD_PrintMsg(HELP_LINE1_TEXT);
          break;
        case 1:
          LCD_PrintMsg(HELP_LINE2_TEXT);
          break;
        case 2:
          LCD_PrintMsg(HELP_LINE3_TEXT);
          break;
        default:
          break;
      }
    }
  }
}

// Public functions ----------------------------------------------------------

 // (C) COPYRIGHT 2010 STMicroelectronics *****END OF FILE***
